"""CLI entry point."""
from __future__ import annotations

import click

from crondag.cli.commands.serve import serve
from crondag.cli.commands.db import db
from crondag.cli.commands.dags import dags
from crondag.cli.commands.runs import runs
from crondag.cli.commands.logs import logs


@click.group()
@click.version_option(package_name="crondag")
def cli():
    """crondag — Lightweight workflow engine."""
    pass


cli.add_command(serve)
cli.add_command(db)
cli.add_command(dags)
cli.add_command(runs)
cli.add_command(logs)
