"""Trigger rules for task execution."""
from __future__ import annotations

import enum


class TriggerRule(str, enum.Enum):
    ALL_SUCCESS = "all_success"
    ALL_DONE = "all_done"
    ONE_FAILED = "one_failed"
